@@ -1,15 +1,18 @@
 import json
-from typing import Type, Set, Dict, Union, Generator, Callable, Any
+from types import MethodType
+from typing import Type, Set, Dict, Union, Generator, Callable, Any, Tuple
 
+from django.db.models import QuerySet
 from django.forms import (
     BaseForm,
     modelformset_factory, HiddenInput,
-    ModelMultipleChoiceField, ModelChoiceField, BaseFormSet,
+    ModelMultipleChoiceField, ModelChoiceField, BaseFormSet, BooleanField, Select, Field,
 )
-from django.http import HttpRequest
+from django.http import HttpRequest, QueryDict
 from django.utils.safestring import mark_safe
+from django.utils.translation import gettext_lazy as _
 
-from .fields import SubformBoundField, SubformField
+from .fields import SubformField, EnhancedBoundField, EnhancedField
 from .formsets import ModelFormSet, SiteformFormSetMixin
 from .utils import bind_subform, UNSET, temporary_fields_patch
 from .widgets import ReadOnlyWidget
@@ -22,6 +25,12 @@ TypeSubform = Union['SiteformsMixin', SiteformFormSetMixin]
 TypeDefFieldsAll = Union[Set[str], str]
 TypeDefSubforms = Dict[str, Type['SiteformsMixin']]
 
+MACRO_ALL = '__all__'
+
+YES_NO_CHOICES = [
+    (True, _('Yes')), (False, _('No'))
+]
+
 
 class SiteformsMixin(BaseForm):
     """Mixin to extend native Django form tools."""
@@ -79,6 +88,7 @@ class SiteformsMixin(BaseForm):
             request: HttpRequest = None,
             src: str = None,
             id: str = '',  # noqa
+            target_url: str = '',
             parent: 'SiteformsMixin' = None,
             hidden_fields: Set[str] = UNSET,
             formset_kwargs: dict = UNSET,
@@ -98,6 +108,9 @@ class SiteformsMixin(BaseForm):
         :param id: Form ID. If defined the form will be rendered
             with this ID. This ID will also be used as auto_id prefix for fields.
 
+        :param target_url: Where form data should be sent.
+            This will appear in 'action' attribute of a 'form' tag.
+
         :param parent: Parent form for a subform.
 
         :param hidden_fields: See the class attribute docstring.
@@ -132,6 +145,7 @@ class SiteformsMixin(BaseForm):
         self.composer_render_form_tag = render_form_tag
 
         self.id = id
+        self.target_url = target_url
 
         if id and 'auto_id' not in kwargs:
             kwargs['auto_id'] = f'{id}_%s'
@@ -157,6 +171,29 @@ class SiteformsMixin(BaseForm):
     def __str__(self):
         return self.render()
 
+    @classmethod
+    def _meta_hook(cls):
+        """Allows hooking on meta construction (see BaseMeta)."""
+
+        subforms = cls.subforms or {}
+        base_fields = cls.base_fields
+
+        for field_name, field in base_fields.items():
+            field: Field
+            # Swap bound field with our custom one
+            # to add .bound_field attr to every widget.
+            field.get_bound_field = MethodType(EnhancedField.get_bound_field, field)
+
+            # Use custom field for subforms.
+            if subforms.get(field_name):
+                base_fields[field_name] = SubformField(
+                    original_field=field,
+                    validators=field.validators,
+                )
+
+    def _preprocess_source_data(self, data: Union[dict, QueryDict]) -> Union[dict, QueryDict]:
+        return data
+
     def _initialize_pre(self, *, args, kwargs):
         # NB: may mutate args and kwargs
 
@@ -172,6 +209,8 @@ class SiteformsMixin(BaseForm):
 
             if is_submitted and request.method == src:
 
+                data = self._preprocess_source_data(data)
+
                 self.data = data
                 files = request.FILES
                 self.files = files
@@ -409,12 +448,12 @@ class SiteformsMixin(BaseForm):
         readonly = self.readonly_fields
         get_readonly_cls = self._get_widget_readonly_cls
 
-        all_macro = '__all__'
+        all_macro = MACRO_ALL
 
         with temporary_fields_patch(self):
 
             for field in self:
-                field: SubformBoundField
+                field: EnhancedBoundField
                 field_name = field.name
                 base_field = field.field
                 instance_field = self.fields[field_name]
@@ -449,3 +488,131 @@ class SiteformsMixin(BaseForm):
             result = callback()
 
         return result
+
+
+class FilteringSiteformsMixin(SiteformsMixin):
+    """Filtering forms base mixin."""
+
+    filtering_rules: Dict[str, str] = {}
+    """Allows setting rules (instructions) to use this form for queryset filtering.
+
+    Example::
+        filtering = {
+            # for these fields we use special lookups
+            'field1': 'icontains',
+            'field2_json_sub': 'sub__gt',
+        }
+
+    """
+
+    filtering_fields_optional: TypeDefFieldsAll = '__all__'
+    """Fields that should be considered optional for filtering. 
+    Use __all__ to describe all fields.
+
+    """
+
+    filtering_fields_choice_undefined: TypeDefFieldsAll = '__all__'
+    """Fields with choices that should include an <undefined> item for filtering. 
+    Use __all__ to describe all fields.
+
+    """
+
+    filtering_choice_undefined_title: str = '----'
+    """Title for choice describing an <undefined> item for filtering."""
+
+    filtering_choice_undefined_value: str = '*'
+    """Value for choice describing an <undefined> item for filtering."""
+
+    @classmethod
+    def _meta_hook(cls):
+        super()._meta_hook()
+
+        all_macro = MACRO_ALL
+        choices_yes_no = YES_NO_CHOICES
+        fields_optional = cls.filtering_fields_optional
+        fields_choice_undef = cls.filtering_fields_choice_undefined
+        undef_choice_title = cls.filtering_choice_undefined_title
+        undef_choice_value = cls.filtering_choice_undefined_value
+
+        base_fields = cls.base_fields.copy()
+        cls.base_fields = base_fields
+
+        for field_name, field in base_fields.items():
+
+            if hasattr(field, '_fltpatched'):
+                # prevent subsequent patching
+                continue
+
+            # todo swap boolean with select to allow <no filtering>
+            # todo note that <undefined> value leads to filtering by False
+            # if isinstance(field, BooleanField):
+            #     choices = choices_yes_no.copy()
+            #     field.choices = choices
+            #     field.widget = Select(choices=choices)
+
+            if fields_optional and (fields_optional == all_macro or field_name in fields_optional):
+                # For proper field rendering.
+                field.widget.is_required = False
+                # For value handling.
+                field.required = False
+
+            if hasattr(field, 'choices') and (fields_choice_undef == all_macro or field_name in fields_choice_undef):
+                field.initial = field.initial or undef_choice_value
+                field.widget.choices.insert(0, (undef_choice_value, undef_choice_title))
+
+            field._fltpatched = True
+
+    def _preprocess_source_data(self, data: Union[dict, QueryDict]) -> Union[dict, QueryDict]:
+        data = super()._preprocess_source_data(data)
+
+        if isinstance(data, QueryDict):
+            data = data.dict()
+
+        undef_choice_value = self.filtering_choice_undefined_value
+
+        # drop undefined values beforehand not to mess with them later
+        data = {
+            key: value
+            for key, value in data.items()
+            if value != undef_choice_value
+        }
+
+        return data
+
+    def filtering_apply(self, queryset: QuerySet) -> Tuple[QuerySet, bool]:
+        """Applies filtering to the queryset using user-submitted
+        data cleaned by this form and filtering instructions (see .filtering) if any.
+
+        Returns a tuple of a query set and boolean:
+
+            * QuerySet
+                * Returns a new filtered queryset if form data is valid.
+                * If user input is invalid (form is not valid) returns initial queryset.
+
+            * Returns True if filters were applied to a query set.
+
+        :param queryset:
+
+        """
+        if not self.is_valid():
+            return queryset, False
+
+        filter_kwargs = {}
+        rules = self.filtering_rules
+        cleaned_data = self.cleaned_data
+
+        for field_name, field in self.fields.items():
+
+            cleaned_value = cleaned_data.get(field_name)
+            if cleaned_value in field.empty_values:
+                continue
+
+            lookup_name = field_name
+            rule = rules.get(field_name)
+
+            if rule:
+                lookup_name = f'{lookup_name}__{rule}'
+
+            filter_kwargs[lookup_name] = cleaned_value
+
+        return queryset.filter(**filter_kwargs), bool(filter_kwargs)
